<?php
echo "<!DOCTYPE html>";
echo "<head>";
echo "<meta charset=\"UTF-8\">";
echo "<title> </title>";
echo "<!--[if lte IE 6]>";
echo "    <script type=\"text/JavaScript\" src=\"../js/jQuery/jquery.js\"></script>";
echo "    <script type=\"text/JavaScript\" src=\"../js/jQuery/jquery.ifixpng.js\"></script>";
echo "    <script type=\"text/JavaScript\">";
echo "    function PNGload(png) {";
echo "        if (typeof(png.ifixpng)==='function') {";
echo "            png.ifixpng('../gfx/blank.gif');";
echo "        }";
echo "    }";
echo "</script>";
echo "<![endif]-->";
echo "</head>";
echo "<body>";

$pngtable = glob('{../gfx/images/*.png,../plugins/*/gfx/*.png}', GLOB_BRACE);
if (is_array($pngtable) && (($total = count($pngtable)) > 0)) {
    echo "<table cellpadding=\"2\" border=\"1\"  CELLSPACING=\"0\">";
    echo "<tr>";
    echo "<td>PNG name</td>";
    echo "<td>16x16</td>";
    echo "<td>32x32</td>";
    echo "<td style=\"background-color:#ccc\">16x16</td>";
    echo "<td style=\"background-color:#ccc\">32x32</td>";
    echo "<td style=\"background-color:#272b30\">16x16</td>";
    echo "<td style=\"background-color:#272b30\">32x32</td>";
    echo "</tr>";
    for ($i = 0; $i < $total; $i++) {
        echo "<tr>";
            echo "<td>".$pngtable[$i]."</td>";
            echo "<!--[if lte IE 6]>";
            echo "<td>";
                echo "<img src=\"".$pngtable[$i]."\" height=\"16\" width=\"16\" onload='PNGload($(this));'/>";
            echo "</td>";
            echo "<td>";
                echo "<img src=\"".$pngtable[$i]."\" height=\"32\" width=\"32\" onload='PNGload($(this));'/>";
            echo "</td>";
            echo "<td style=\"background-color:#ccc\">";
                echo "<img src=\"".$pngtable[$i]."\" height=\"16\" width=\"16\" onload='PNGload($(this));'/>";
            echo "</td>";
            echo "<td style=\"background-color:#ccc\">";
                echo "<img src=\"".$pngtable[$i]."\" height=\"32\" width=\"32\" onload='PNGload($(this));'/>";
            echo "</td>";
            echo "<td style=\"background-color:#272b30\">";
                echo "<img src=\"".$pngtable[$i]."\" height=\"16\" width=\"16\" onload='PNGload($(this));'/>";
            echo "</td>";
            echo "<td style=\"background-color:#272b30\">";
                echo "<img src=\"".$pngtable[$i]."\" height=\"32\" width=\"32\" onload='PNGload($(this));'/>";
            echo "</td>";
            echo "<![endif]-->";
            echo "<!--[if gt IE 6]><!-->";
            echo "<td>";
                echo "<img src=\"".$pngtable[$i]."\" height=\"16\" width=\"16\"/>";
            echo "</td>";
            echo "<td>";
                echo "<img src=\"".$pngtable[$i]."\" height=\"32\" width=\"32\"/>";
            echo "</td>";
            echo "<td style=\"background-color:#ccc\">";
                echo "<img src=\"".$pngtable[$i]."\" height=\"16\" width=\"16\"/>";
            echo "</td>";
            echo "<td style=\"background-color:#ccc\">";
                echo "<img src=\"".$pngtable[$i]."\" height=\"32\" width=\"32\"/>";
            echo "</td>";
            echo "<td style=\"background-color:#272b30\">";
                echo "<img src=\"".$pngtable[$i]."\" height=\"16\" width=\"16\"/>";
            echo "</td>";
            echo "<td style=\"background-color:#272b30\">";
                echo "<img src=\"".$pngtable[$i]."\" height=\"32\" width=\"32\"/>";
            echo "</td>";
            echo "<!--<![endif]-->";
        echo "</tr>";
    }
    echo "</table>";
}
echo "</body>";
