/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * DataTables sorting
 */
.sorting_asc {
    background: url('../../gfx/sort_asc.gif') no-repeat center right;
}

.sorting_desc {
    background: url('../../gfx/sort_desc.gif') no-repeat center right;
}

.sorting {
    background: url('../../gfx/sort_both.gif') no-repeat center right;
}