<?php echo"<?xml version=\"1.0\" encoding=\"utf-8\"?>"; ?>
<!DOCTYPE html PUBLIC "-//W3C//DTD  XHTML 1.0  Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <meta http-equiv="Content-Script-Type" content="text/javascript" />
    <meta http-equiv="Content-Style-Type" content="text/css" />
    <link rel="shortcut icon" href="gfx/favicon.gif" />
    <link type="text/css" rel="stylesheet" href="./templates/phpsysinfo.css" />
    <title>phpSysInfo - Error</title>
    <noscript>
      <meta http-equiv="refresh" content="2; URL=index.php?disp=static" />
    </noscript>
</head>
<body onload="doRedirect()">
<h1>phpSysInfo - Error</h1>
<div style="position:absolute;top:150px;text-align:center;width:95%;">
    <p style="margin:12pt;"><strong>phpsysinfo.ini</strong> does not exist or is not readable by the webserver in the phpsysinfo directory or is misunderstood.</p>
    <p style="margin:12pt;">Generated by&nbsp;<a href="http://phpsysinfo.sourceforge.net/">phpSysInfo&nbsp;-&nbsp;<?php echo PSI_VERSION_STRING ?></a></p>
</div>
</body>
</html>
