body {
    font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
    font-size: 13px;
    padding-top: 80px;
    color: #333;
    background-color: #fff;
}

wbr {
    display: inline-block;
}

.table td.rightCell, .table th.rightCell {
    text-align: right;
}

.percent {
    text-align: center;
}

.card {
    border-color: #f2f2f2;
}

.navbar {
    background-color: #3b87c8;
    border-color: #2d6da3;
}

.navbar-brand, .navbar-brand:hover {
    color: #fff;
    margin-right: 0;
}

.treegrid-span {
    display: table-cell;
}

.treegrid-spanbold {
    font-weight: bold;
    display: table-cell;
}

.treegrid-indent {
    width: 16px;
    height: 16px;
    display: table-cell;
}

.treegrid-expander {
    width: 0px;
    height: 16px;
    display: table-cell;
    cursor: pointer;
}

.normalicon {
    width: 16px;
    height: 16px;
    position: relative;
    top: 1px;
    display: table-cell;
    font-style: normal;
    font-weight: normal;
    line-height: 1;

    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
}
.normalicon-right:before {
    content: "\25ba";
}
.normalicon-down:before {
    content: "\25bc";
}

table.sortable thead th {
    cursor: pointer;
    position: relative;
    top: 0;
    left: 0;
}

table.sortable thead th:hover {
    background-color: #ececec;
}

.sorttable_nosort {
    cursor: auto!important;
}

.table-hover > tbody > tr:hover {
    color: #333;
    background-color: #ececec;
}

.table-hover > tfoot > tr:hover {
    background-color: #ececec;
}

table.borderless td, table.borderless th, table.noborderattop tr:first-child td, table.noborderattop tr:first-child th {
    border: none!important;
}

.table {
    color: #333;
    background-color: #fff!important;
    margin: 0;
    border: 0;
}

.logo {
    cursor: pointer;
    width: 32px;
}

.select {
    color: #333;
}

select {
    background-color: #fefefe;
}

.template {
    cursor: pointer;
    color: #333;
}

.language {
    cursor: pointer;
    color: #333;
}

.card-header {
    background-color: #337ab7!important;
    border-color: #337ab7!important;
    color: #fff!important;
}

.progress {
    margin-bottom: 0;
    background-color: #f5f5f5;
  -webkit-box-shadow: inset 0 1px 2px rgba(0, 0, 0, .1);
          box-shadow: inset 0 1px 2px rgba(0, 0, 0, .1);
}

.progress-bar-success {
    background-color: #5cb85c;
}

.progress-bar-info {
    background-color: #5bc0de;
}

.progress-bar-warning {
    background-color: #f0ad4e;
}

.progress-bar-danger {
    background-color: #d9534f;
}

.modal-content {
    background-color: #fff;
}

.list-group {
    display: block;
}

.list-group-item {
    background-color: #fff;
}

.reload {
    background-image: url("../gfx/reload.gif");
    float: right;
    cursor: pointer;
    width: 16px;
    height: 16px;
}

.container {
    padding:0;
}

.table thead th {
    border-top: none;
}

a, a:visited {
    color: #000;
}

a:hover {
    text-decoration: underline;
    color: #000;
}

.psihref, .psihref:visited {
    color: #fff;
}

.psihref:hover {
    text-decoration: underline;
    color: #fff;
}

.col-lg-6, .col-lg-12 {
    padding-bottom: 20px;
}

.badge {
    font-size: 12px;
    border-radius: 10px;
    padding: 3px 7px;
    float: right;
}

.table-nopadding > tbody > tr > td,
.table-nopadding > tbody > tr > th {
  padding: 0;
}
