<?php echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>"; ?>
<!DOCTYPE html PUBLIC "-//W3C//DTD  XHTML 1.0  Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <meta http-equiv="Content-Script-Type" content="text/javascript" />
    <meta http-equiv="Content-Style-Type" content="text/css" />
    <link rel="shortcut icon" href="gfx/favicon.gif" />
    <link type="text/css" rel="stylesheet" href="./templates/phpsysinfo.css" />
    <title>phpSysInfo - Redirection</title>
    <noscript>
      <meta http-equiv="refresh" content="2; URL=index.php?disp=static" />
    </noscript>
    <script type="text/JavaScript" language="JavaScript">
      <!--
      var sTargetURL = "index.php?disp=static";
      //-->
    </script>
    <script type="text/JavaScript" language="JavaScript1.1">
      <!--
      sTargetURL = "index.php?disp=bootstrap";
      //-->
    </script>
    <!--[if lt IE 8]>
        <script type="text/JavaScript" language="JavaScript">
            sTargetURL = "index.php?disp=dynamic";
        </script>
    <![endif]-->
    <script type="text/JavaScript" language="JavaScript">
      <!--
      function doRedirect() {
        setTimeout( "window.location.href = sTargetURL", 2*1000 );
      }
      //-->
    </script>
    <script type="text/JavaScript" language="JavaScript1.1">
      <!--
      function doRedirect() {
        window.location.replace( sTargetURL );
      }
      doRedirect();
      //-->
    </script>
</head>
<body onload="doRedirect()">
<h1>REDIRECTING ... </h1>
<div style="position:absolute;top:150px;text-align:center;width:95%;">
    <p style="margin:12pt;">Loading <a href="index.php?disp=static">redirection target</a></p>
    <p style="margin:12pt;">In approx. 2 seconds the redirection target page should load.<br/>
    If it doesn't please select the link above.</p>
    <p style="margin:12pt;">Generated by&nbsp;<a href="http://phpsysinfo.sourceforge.net/">phpSysInfo&nbsp;-&nbsp;<?php echo PSI_VERSION_STRING ?></a></p>
</div>
</body>
</html>
