<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta http-equiv="Content-Security-Policy" content="default-src 'unsafe-inline' 'unsafe-eval' 'self'">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="apple-mobile-web-app-capable" content="yes">
    <meta name="mobile-web-app-capable" content="yes">
    <meta name="description" content="phpSysInfo is a customizable PHP script that displays information about your system nicely">
    <meta name="author" content="phpSysInfo organization at GitHub">
    <title>phpSysInfo <?php echo PSI_VERSION_STRING ?></title>
    <link href="gfx/favicon.ico" type="image/x-icon" rel="shortcut icon">
    <noscript><meta http-equiv="refresh" content="2; URL=index.php?disp=static"/></noscript>
    <link rel="stylesheet" type='text/css' href="templates/vendor/bootstrap.min.css">
    <link rel="stylesheet" type='text/css' href="templates/vendor/bootstrap-webapp.css">
    <!--[if IE 9]>
        <link rel="stylesheet" type='text/css' href="templates/vendor/bootstrap-ie9.css">
        <script type="text/JavaScript" src="./js.php?name=bootstrap-ie9"></script>
    <![endif]-->
    <!--[if IE 8]>
        <link rel="stylesheet" type='text/css' href="templates/vendor/bootstrap-ie8.css">
        <script type="text/JavaScript" src="./js.php?name=html5shiv-printshiv"></script>
        <script type="text/JavaScript" src="./js.php?name=bootstrap-ie8"></script>
    <![endif]-->
    <!--[if lt IE 8]>
        <script type="text/javascript">
            alert('You are using an unsupported browser. Update Internet Explorer to at least version 8');
            die();
        </script>
    <![endif]-->
    <link rel="stylesheet" type='text/css' href="templates/misc/emptyfile.css" id="PSI_CSS_Fix">
    <link rel="stylesheet" type='text/css' href="templates/misc/emptyfile.css" id="PSI_Template">
</head>

<body>

<input type="hidden" id="plugins" value="<?php $plugins = ''; foreach (CommonFunctions::getPlugins() as $plugin) : $plugins .= $plugin.','; endforeach; echo substr($plugins, 0, -1);?>"/>
<input type="hidden" id="showCPUListExpanded" value="<?php echo $showCPUListExpanded;?>"/>
<input type="hidden" id="showCPUInfoExpanded" value="<?php echo $showCPUInfoExpanded;?>"/>
<input type="hidden" id="showNetworkInfosExpanded" value="<?php echo $showNetworkInfosExpanded;?>"/>
<input type="hidden" id="showNetworkActiveSpeed" value="<?php echo $showNetworkActiveSpeed;?>"/>
<input type="hidden" id="showCPULoadCompact" value="<?php echo $showCPULoadCompact;?>"/>
<input type="hidden" id="hideBootstrapLoader" value="<?php echo $hideBootstrapLoader;?>"/>
<input type="hidden" id="hideTotals" value="<?php echo $hideTotals;?>"/>
<input type="hidden" id="blocks" value="<?php echo $blocks;?>"/>

<div class="navbar fixed-top">
    <div class="container" style="display:block;">
        <div style="display:table-cell">
            <div class="navbar-brand">
                <img class="logo" alt="" src="gfx/logo_32.gif" title="reload"/>&thinsp;phpSys<a title="phpSysInfo - <?php echo PSI_VERSION_STRING; ?>" class="psihref" href="http://phpsysinfo.sourceforge.net/" target="psihref">Info</a><div class="errorbutton" id="errorbutton" title="errors" data-target="#errors-dialog" style="display:inline-block;background:url('gfx/attention.gif') no-repeat center;width:49px;height:32px;visibility:hidden;">
                    <img id="loader" alt="" src="gfx/ajax-loader.gif" style="margin:0 3px 0 3px;visibility:hidden;"/>
                </div>
            </div>
        </div>

        <div id="select" class="select" style="display:none;width:100%;vertical-align:middle;text-align:right;">
            <div id="tempblock" style="display:none;">
                <span class="lang_044">Template</span>
                <select class="template" id="template">
<?php if ($picktemplate) {
foreach ($bootstraptemplates as $t) :
    $selected = "";
    if ($bootstraptemplate === $t) {
        $selected = " selected=\"selected\"";
    }
    echo "                    <option value=\"".$t."\"".$selected.">".$t."</option>\n";
endforeach;
} else {
    echo "                    <option value=\"".$bootstraptemplate."\" selected=\"selected\">".$bootstraptemplate."</option>\n";
}?>
                </select>
            </div>
            <div id="langblock" style="display:none;">
                <span class="lang_045">Language</span>
                <select class="language" id="language">
<?php if ($picklanguage) {
foreach ($languages as $l) :
    $selected = "";
    if ($language === $l) {
        $selected = " selected=\"selected\"";
    }
    echo "                    <option value=\"".$l."\"".$selected.">".$l."</option>\n";
endforeach;
} else {
    echo "                    <option value=\"".$language."\" selected=\"selected\">".$language."</option>\n";
}?>
                </select>
            </div>
        </div>

        <noscript>
            <p>Your navigator does not support JavaScript (or JavaScript is not activated).</p>
            <p>In approximatively 2 seconds you will be redirected to the static version of phpSysInfo.</p>
            <p>&nbsp;</p>
            <p><a href="index.php?disp=static">Click here to switch to the static version</a></p>
        </noscript>
    </div>
</div>

<div class="container">
<div id="output" class="row" style="display:none;">
    <div id="block_vitals" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_002">System vitals</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="vitals" class="table table-hover table-sm noborderattop">
                        <tbody>
                            <tr>
                                <th><span class="lang_003">Hostname</span></th>
                                <td><span data-bind="Hostname"></span></td>
                            </tr>
                            <tr>
                                <th><span class="lang_004">Listening IP</span></th>
                                <td><span data-bind="IPAddr"></span></td>
                            </tr>
                            <tr>
                                <th><span class="lang_005">Kernel Version</span></th>
                                <td><span data-bind="Kernel"></span></td>
                            </tr>
                            <tr>
                                <th><span class="lang_006">Distro Name</span></th>
                                <td><span data-bind="Distro"></span></td>
                            </tr>
                            <tr>
                                <th><span class="lang_127">OS Type</span></th>
                                <td><span data-bind="OS"></span></td>
                            </tr>
                            <tr>
                                <th><span class="lang_007">Uptime</span></th>
                                <td><span data-bind="Uptime"></span></td>
                            </tr>
                            <tr>
                                <th><span class="lang_095">Last boot</span></th>
                                <td><span data-bind="LastBoot"></span></td>
                            </tr>
                            <tr>
                                <th><span class="lang_008">Current Users</span></th>
                                <td><span data-bind="Users"></span></td>
                            </tr>
                            <tr id="tr_LoadAvg">
                                <th><span class="lang_009">Load Averages</span></th>
                                <td><span data-bind="LoadAvg"></span></td>
                            </tr>
                            <tr id="tr_SysLang">
                                <th><span class="lang_097">System Language</span></th>
                                <td><span data-bind="SysLang"></span></td>
                            </tr>
                            <tr id="tr_CodePage">
                                <th><span class="lang_098">Code Page</span></th>
                                <td><span data-bind="CodePage"></span></td>
                            </tr>
                            <tr id="tr_Processes">
                                <th><span class="lang_110">Processes</span></th>
                                <td><span data-bind="Processes"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_hardware" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_010">Hardware Information</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="hardware" class="table table-hover table-sm noborderattop">
                        <tbody id="hardware-data">
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_memory" class="col-lg-12" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_027">Memory Usage</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="memory" class="table table-hover table-sm">
                        <thead>
                            <tr>
                                <th><span class="lang_034">Type</span></th>
                                <th><span class="lang_033">Usage</span></th>
                                <th class="rightCell"><span class="lang_125">Free</span></th>
                                <th class="rightCell"><span class="lang_036">Used</span></th>
                                <th class="rightCell"><span class="lang_037">Size</span></th>
                            </tr>
                        </thead>
                        <tbody id="memory-data">
                            <tr>
                                <th><span data-bind="Type"></span></th>
                                <td><span data-bind="Usage"></span></td>
                                <td class="rightCell"><span data-bind="Free"></span></td>
                                <td class="rightCell"><span data-bind="Used"></span></td>
                                <td class="rightCell"><span data-bind="Total"></span></td>
                            </tr>
                        </tbody>
                        <tfoot id="swap-data" style="display:none;">
                            <tr>
                                <th><span data-bind="Name"></span></th>
                                <td><span data-bind="Usage"></span></td>
                                <td class="rightCell"><span data-bind="Free"></span></td>
                                <td class="rightCell"><span data-bind="Used"></span></td>
                                <td class="rightCell"><span data-bind="Total"></span></td>
                            </tr>
                        </tfoot>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_filesystem" class="col-lg-12" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_030">Mounted Filesystems</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="filesystem" class="table table-hover table-sm sortable">
                        <thead>
                            <tr>
                                <th id="filesystem_MountPoint"><span class="lang_031">Mountpoint</span></th>
                                <th><span class="lang_034">Type</span></th>
                                <th><span class="lang_032">Partition</span></th>
                                <th><span class="lang_033">Usage</span></th>
                                <th class="rightCell sorttable_numeric"><span class="lang_035">Free</span></th>
                                <th class="rightCell sorttable_numeric"><span class="lang_036">Used</span></th>
                                <th class="rightCell sorttable_numeric"><span class="lang_037">Size</span></th>
                            </tr>
                        </thead>
                        <tbody id="filesystem-data">
                            <tr>
                                <th><span data-bind="MountPoint"></span></th>
                                <td><span data-bind="FSType"></span></td>
                                <td><span data-bind="Name"></span></td>
                                <td><span data-bind="Percent"></span></td>
                                <td class="rightCell"><span data-bind="Free"></span></td>
                                <td class="rightCell"><span data-bind="Used"></span></td>
                                <td class="rightCell"><span data-bind="Total"></span></td>
                            </tr>
                        </tbody>
                        <tfoot id="filesystem-foot" style="display:none;">
                            <tr>
                                <td></td>
                                <td></td>
                                <th><span class="lang_038">Totals</span></th>
                                <th><span data-bind="Percent"></span></th>
                                <th class="rightCell"><span data-bind="Free"></span></th>
                                <th class="rightCell"><span data-bind="Used"></span></th>
                                <th class="rightCell"><span data-bind="Total"></span></th>
                            </tr>
                        </tfoot>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_network" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_021">Network Interface</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="network" class="table table-hover table-sm">
                        <thead>
                            <tr>
                                <th style="width:60%"><span class="lang_022">Device</span></th>
                                <th class="rightCell"><span class="lang_023">Receive</span></th>
                                <th class="rightCell"><span class="lang_024">Send</span></th>
                                <th class="rightCell"><span class="lang_025">Err/<wbr>Drop</span></th>
                            </tr>
                        </thead>
                        <tbody id="network-data"></tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_voltage" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_052">Voltage</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="voltage" class="table table-hover table-sm">
                        <thead>
                            <tr>
                                <th><span class="lang_059">Label</span></th>
                                <th class="rightCell"><span class="lang_054">Value</span></th>
                                <th class="rightCell"><span class="lang_055">Min</span></th>
                                <th class="rightCell"><span class="lang_056">Max</span></th>
                            </tr>
                        </thead>
                        <tbody id="voltage-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Min"></span></td>
                                <td class="rightCell"><span data-bind="Max"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_current" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_105">Current</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="current" class="table table-hover table-sm">
                        <thead>
                            <tr>
                                <th><span class="lang_059">Label</span></th>
                                <th class="rightCell"><span class="lang_054">Value</span></th>
                                <th class="rightCell"><span class="lang_055">Min</span></th>
                                <th class="rightCell"><span class="lang_056">Max</span></th>
                            </tr>
                        </thead>
                        <tbody id="current-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Min"></span></td>
                                <td class="rightCell"><span data-bind="Max"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_temperature" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_051">Temperatures</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="temperature" class="table table-hover table-sm">
                        <thead>
                            <tr>
                                <th><span class="lang_059">Label</span></th>
                                <th class="rightCell"><span class="lang_054">Value</span></th>
                                <th class="rightCell"><span class="lang_058">Limit</span></th>
                            </tr>
                        </thead>
                        <tbody id="temperature-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Max"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_fans" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_053">Fans</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="fans" class="table table-hover table-sm">
                        <thead>
                            <tr>
                                <th><span class="lang_059">Label</span></th>
                                <th class="rightCell"><span class="lang_054">Value</span></th>
                                <th class="rightCell"><span class="lang_055">Min</span></th>
                            </tr>
                        </thead>
                        <tbody id="fans-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Min"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_power" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_102">Power</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="power" class="table table-hover table-sm">
                        <thead>
                            <tr>
                                <th><span class="lang_059">Label</span></th>
                                <th class="rightCell"><span class="lang_054">Value</span></th>
                                <th class="rightCell"><span class="lang_058">Limit</span></th>
                            </tr>
                        </thead>
                        <tbody id="power-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Max"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_other" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_121">Other</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="other" class="table table-hover table-sm">
                        <thead>
                            <tr>
                                <th><span class="lang_059">Label</span></th>
                                <th class="rightCell"><span class="lang_054">Value</span></th>
                            </tr>
                        </thead>
                        <tbody id="other-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_ups" class="col-lg-6" style="display:none;">
        <div class="card">
            <div class="card-header"><span class="lang_068">UPS Information</span></div>
            <div class="card-body">
                <div class="table-responsive">
                    <table id="ups" class="table table-hover table-sm noborderattop">
                        <tbody id="ups-data">
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

<?php foreach (CommonFunctions::getPlugins() as $plugin) :
if (file_exists(PSI_APP_ROOT . '/plugins/' . $plugin . '/' . $plugin . '_bootstrap.html')) : ?>
<?php include PSI_APP_ROOT. '/plugins/' . $plugin . '/' . $plugin . '_bootstrap.html'; ?>

<?php endif; endforeach; ?>

</div>

<!-- Errors details modal dialogs -->
<div class="modal fade" id="errors-dialog" tabindex="-1" role="dialog" aria-hidden="true">
    <div class="modal-dialog modal-lg">
        <div class="modal-content">
            <div class="modal-header">
                <h4 class="modal-title">Errors</h4>
                <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                    <span aria-hidden="true">&times;</span>
                </button>
            </div>
            <div class="modal-body">
                <ul id="errors" class="list-group">
                </ul>
            </div>
        </div>
        <!-- /.modal-content -->
    </div>
    <!-- /.modal-dialog -->
</div>
<!-- /.modal -->

</div>

<script type="text/JavaScript" src="./js.php?name=jquery"></script>
<script type="text/JavaScript" src="./js.php?name=console-shim"></script>
<script type="text/JavaScript" src="./js.php?name=transparency"></script>
<script type="text/JavaScript" src="./js.php?name=sorttable"></script>
<script type="text/JavaScript" src="./js.php?name=jquery.treegrid"></script>
<script type="text/JavaScript" src="./js.php?name=bootstrap-util"></script>
<script type="text/JavaScript" src="./js.php?name=bootstrap-modal"></script>

<?php foreach (CommonFunctions::getPlugins() as $plugin) : ?>
<script type="text/JavaScript" src="./js.php?plugin=<?php echo $plugin ?>&amp;name=<?php echo $plugin ?>_bootstrap"></script>
<?php endforeach; ?>

<script type="text/JavaScript" src="./js.php?name=phpsysinfo_bootstrap"></script>

</body>
</html>
