/* NextGen Remixed by iDash.pl */
a {
    text-decoration: none;
    color: #5a7000;
}

a:hover {
    text-decoration: underline;
    color: #5a7000;
}

* {
    margin: 0;
    padding: 0;
}

wbr {
    display: inline-block;
}

html {
    height: 100%;
    background: url("idash/bg.png") repeat-x scroll center top #111;
}

body {
    position: relative;
    width: 940px;
    _width: 945px; /* ie6 */
    min-height: 100%;
    overflow: auto;
    margin: 0 auto;
    padding: 20px 20px 0 20px;
    font: 0.75em tahoma, arial, sans-serif;
    color: #ccc;
}

div#output {
    margin: -20px -10px 0 -10px;
    padding: 95px 0 0 0;
}

p {
    padding: 4px 10px 2px 10px;
    line-height: 1.6;
    text-align: left;
    vertical-align: top;
}

h1 {
    position: absolute;
    top: 35px;
    left: 10px;
    margin: 0;
    padding: 0px 10px;
    font-size: 2em;
    font-weight: normal;
    color: #fff;
}

h2 {
    padding: 5px 10px;
    font-family: "trebuchet ms";
    font-size: 1.2em;
    font-weight: bold;
    letter-spacing: 0.0em;
    text-transform: uppercase;
    color: #fff; /* #7d9100; */
    background: #354242;
}

th, td, h3 {
    padding: 4px 10px 2px 10px;
    text-align: left;
    vertical-align: top;
}

h3 {
    font-size: 120%;
}

table {
    width: 100%;
}

select {
    background-color: #fefefe;
}

#select {
    position: absolute;
    top: 75px;
    color: #fff;
    right: 30px;
    width: 370px;
    text-align: right;
    padding: 0 10px 0 10px;
}

#select select {
    width: 100px;
}

.fullsize, .halfsize {
    float: left;
    margin: 10px 0 0 10px;
    _margin: 10px 5px 0 5px; /* ie6 */
    padding: 1px;
    border: 1px solid #354242;
}

.fullsize {
    width: 916px;
}

.halfsize {
    width: 451px;
}

#filesystemTable thead tr th {
    cursor: pointer;
}

#footer {
    color: #777;
    clear: both;
    margin: 12px;
    padding: 13px 25px;
    line-height: 18px;
    text-align: right;
}

.lang_047 {
    color:444
}

.plugin {
    float: left;
    margin: 10px 0 0 10px;
    _margin: 10px 5px 0 5px; /* ie6 */
    padding: 1px;
    border: 1px solid #354242;
}

.even {
    background: #333;
}

.bar {
    background: #34da64 url("idash/html.gif");
}

.barwarn {
    background: #e69575 url("idash/htmlwarn.gif");
}

.barrest {
    background: #57da34 url("idash/htmlrest.gif");
}

.right {
    text-align: right;
    padding-right: 20px;
}

.dataTables_wrapper{
    margin: 0 0 0 0 !important;
    border: 0px !important;
}

.treeimg {
    display: table-cell;
    *float: left; /* ie6 ie7 */
    vertical-align: top;
}

.treespan {
    display: table-cell;
    *float: left; /* ie6 ie7 */
}

.treespanbold {
    font-weight: bold;
    display: table-cell;
    *float: left; /* ie6 ie7 */
}

.tree tr td {
    cursor: auto;
    height: 100%;
}

.treediv {
    display: table;
    height: 100%;
}

.tablemain {
    width: 100%;
    height: 100%;
}
