<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <meta http-equiv="Content-Style-Type" content="text/css" />
    <meta http-equiv="Content-Script-Type" content="text/javascript" />
    <meta http-equiv="Content-Security-Policy" content="default-src 'unsafe-inline' 'unsafe-eval' 'self'" />
    <meta name="Description" content="phpSysInfo is a customizable PHP script that displays information about your system nicely" />
    <noscript>
      <meta http-equiv="refresh" content="2; URL=index.php?disp=static" />
    </noscript>
    <link rel="shortcut icon" href="gfx/favicon.gif" />
    <link type="text/css" rel="stylesheet" href="./templates/misc/emptyfile.css" title="PSI_Template" />
    <link type="text/css" rel="stylesheet" href="./templates/plugin/nyroModal.full.css" />
    <link type="text/css" rel="stylesheet" href="./templates/plugin/jquery.jgrowl.css" />
    <link type="text/css" rel="stylesheet" href="./templates/plugin/jquery.dataTables.css" />
    <link type="text/css" rel="stylesheet" href="./templates/plugin/jquery.treeTable.css" />

    <script type="text/JavaScript" src="./js.php?name=jquery"></script>
    <script type="text/JavaScript" src="./js.php?name=jquery.dataTables"></script>
    <script type="text/JavaScript" src="./js.php?name=jquery.nyroModal"></script>
    <script type="text/JavaScript" src="./js.php?name=jquery.jgrowl"></script>
    <script type="text/JavaScript" src="./js.php?name=jquery.timer"></script>
    <script type="text/JavaScript" src="./js.php?name=jquery.treeTable"></script>
<!--[if lte IE 6]>
    <script type="text/JavaScript" src="./js.php?name=jquery.ifixpng"></script>
<![endif]-->
    <script type="text/JavaScript" src="./js.php?name=phpsysinfo"></script>
<?php foreach (CommonFunctions::getPlugins() as $plugin) : ?>
    <?php if (file_exists("./plugins/".$plugin."/css/".$plugin.".css")) : ?>
      <link type="text/css" rel="stylesheet" href="./plugins/<?php echo $plugin ?>/css/<?php echo $plugin ?>.css" />
    <?php endif; ?>
    <script type="text/JavaScript" src="./js.php?plugin=<?php echo $plugin ?>"></script>
<?php endforeach; ?>
    <title>phpSysInfo <?php echo PSI_VERSION_STRING ?></title>
  </head>
  <body>
    <input type="hidden" id="showCPUListExpanded" value="<?php echo $showCPUListExpanded;?>"/>
    <input type="hidden" id="showCPUInfoExpanded" value="<?php echo $showCPUInfoExpanded;?>"/>
    <input type="hidden" id="showNetworkInfosExpanded" value="<?php echo $showNetworkInfosExpanded;?>"/>
    <input type="hidden" id="showMemoryInfosExpanded" value="<?php echo $showMemoryInfosExpanded;?>"/>
    <input type="hidden" id="showNetworkActiveSpeed" value="<?php echo $showNetworkActiveSpeed;?>"/>
    <input type="hidden" id="showCPULoadCompact" value="<?php echo $showCPULoadCompact;?>"/>
    <input type="hidden" id="hideTotals" value="<?php echo $hideTotals;?>"/>
    <input type="hidden" id="increaseWidth" value="<?php echo $increaseWidth;?>"/>
    <input type="hidden" id="blocks" value="<?php echo $blocks;?>"/>
    <div id="loader">
      <h1>
        <a href="#errors" class="nyroModal">
          <img id="loadwarn" style="vertical-align: middle; display:none; border:0px; width:32px;" src="./gfx/attention.gif" title="errors" alt="errors" />
        </a>
      Loading... please wait!</h1>
      <div id="container">
        <noscript>
        <p>Your navigator does not support JavaScript (or JavaScript is not activated).</p>
        <p>In approximatively 2 seconds you will be redirected to the static version of phpSysInfo.</p>
        <p>&nbsp;</p>
        <p><a href="index.php?disp=static">Click here to switch to the static version</a></p>
        </noscript>
      </div>
    </div>
    <div id="errors" style="display:none; width:940px">
      <div id="errorlist">
        <h2>Oh, I'm sorry. Something seems to be wrong.</h2>
      </div>
    </div>
    <div id="output" style="display:none;">
      <h1>
        <a href="#errors" class="nyroModal">
          <img id="warn" style="vertical-align: middle; display:none; border:0px; width:32px;" src="./gfx/attention.gif" title="errors" alt="errors" />
        </a>
        <span id="title">
          <span class="lang_001">System information</span>
          :&nbsp;<span id="s_hostname_title"></span>
          (<span id="s_ip_title"></span>)
        </span>
      </h1>
      <div id="select">
        <span class="lang_044" style="display:none;">Template</span>
        <select id="template" name="template" style="display:none;">
<?php
if ($picktemplate) {
    foreach ($templates as $t) :
        $selected = "";
        if ($template === $t) {
            $selected = " selected=\"selected\"";
        }
        echo "          <option value=\"".$t."\"".$selected.">".$t."</option>\n";
    endforeach;
} else {
    echo "          <option value=\"".$template."\" selected=\"selected\">".$template."</option>\n";
}
?>
        </select>
        <span class="lang_045" style="display:none;">Language</span>
        <select id="language" name="language" style="display:none;">
<?php
if ($picklanguage) {
    foreach ($languages as $l) :
        $selected = "";
        if ($language === $l) {
            $selected = " selected=\"selected\"";
        }
        echo "          <option value=\"".$l."\"".$selected.">".$l."</option>\n";
    endforeach;
} else {
    echo "          <option value=\"".$language."\" selected=\"selected\">".$language."</option>\n";
}
?>
        </select>
      </div>
      <div id="vitals" class="halfsize">
        <h2><span class="lang_002">System vitals</span></h2>
        <div style="overflow-x:auto;">
          <table id="vitalsTable" style="border-collapse:collapse;">
            <tr>
              <td style="width:160px;"><span class="lang_003">Hostname</span></td>
              <td><span id="s_hostname"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_004">Listening IP</span></td>
              <td><span id="s_ip"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_005">Kernel Version</span></td>
              <td><span id="s_kernel"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_006">Distro Name</span></td>
              <td><span id="s_distro"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_127">OS Type</span></td>
              <td><span id="s_os"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_007">Uptime</span></td>
              <td><span id="s_uptime"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_095">Last boot</span></td>
              <td><span id="s_lastboot"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_008">Current Users</span></td>
              <td><span id="s_users"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_009">Load Averages</span></td>
              <td><span id="s_loadavg"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_097">System Language</span></td>
              <td><span id="s_syslang"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_098">Code Page</span></td>
              <td><span id="s_codepage"></span></td>
            </tr>
            <tr>
              <td style="width:160px;"><span class="lang_110">Processes</span></td>
              <td><span id="s_processes"></span></td>
            </tr>
          </table>
        </div>
      </div>
      <div id="hardware" class="halfsize">
      </div>
      <div id="memory" class="fullsize">
      </div>
      <div id="filesystem" class="fullsize">
      </div>
      <div id="network" class="halfsize">
      </div>
      <div id="voltage" class="halfsize" style="display:none;">
        <h2><span class="lang_052">Voltage</span></h2>
        <div style="overflow-x:auto;">
          <table class="stripeMe" id="voltageTable" style="border-collapse:collapse;">
            <thead>
              <tr>
                <th><span class="lang_059">Label</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_054">Value</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_055">Min</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_056">Max</span></th>
              </tr>
            </thead>
            <tbody>
            </tbody>
          </table>
        </div>
      </div>
      <div id="current" class="halfsize" style="display:none;">
        <h2><span class="lang_105">Current</span></h2>
        <div style="overflow-x:auto;">
          <table class="stripeMe" id="currentTable" style="border-collapse:collapse;">
            <thead>
              <tr>
                <th><span class="lang_059">Label</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_054">Value</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_055">Min</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_056">Max</span></th>
              </tr>
            </thead>
            <tbody>
            </tbody>
          </table>
        </div>
      </div>
      <div id="temperature" class="halfsize" style="display:none;">
        <h2><span class="lang_051">Temperature</span></h2>
        <div style="overflow-x:auto;">
          <table class="stripeMe" id="temperatureTable" style="border-collapse:collapse;">
            <thead>
              <tr>
                <th><span class="lang_059">Label</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_054">Value</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_058">Limit</span></th>
              </tr>
            </thead>
            <tbody>
            </tbody>
          </table>
        </div>
      </div>
      <div id="fans" class="halfsize" style="display:none;">
        <h2><span class="lang_053">Fans</span></h2>
        <div style="overflow-x:auto;">
          <table class="stripeMe" id="fansTable" style="border-collapse:collapse;">
            <thead>
              <tr>
                <th><span class="lang_059">Label</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_054">Value</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_055">Min</span></th>
              </tr>
            </thead>
            <tbody>
            </tbody>
          </table>
        </div>
      </div>
      <div id="power" class="halfsize" style="display:none;">
        <h2><span class="lang_102">Power</span></h2>
        <div style="overflow-x:auto;">
          <table class="stripeMe" id="powerTable" style="border-collapse:collapse;">
            <thead>
              <tr>
                <th><span class="lang_059">Label</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_054">Value</span></th>
                <th class="right" style="width:24.3%;"><span class="lang_058">Limit</span></th>
              </tr>
            </thead>
            <tbody>
            </tbody>
          </table>
        </div>
      </div>
      <div id="other" class="halfsize" style="display:none;">
        <h2><span class="lang_121">Other</span></h2>
        <div style="overflow-x:auto;">
          <table class="stripeMe" id="otherTable" style="border-collapse:collapse;">
            <thead>
              <tr>
                <th><span class="lang_059">Label</span></th>
                <th class="right" style="width:48.6%;"><span class="lang_054">Value</span></th>
              </tr>
            </thead>
            <tbody>
            </tbody>
          </table>
        </div>
      </div>
      <div id="ups" class="halfsize" style="display:none;">
      </div>
      <div id="footer">
        <span class="lang_047">Generated by</span>&nbsp;<a href="http://phpsysinfo.sourceforge.net/" target="psihref">phpSysInfo&nbsp;-&nbsp;<span id="version"></span></a>
      </div>
    </div>
  </body>
</html>
