a {
    text-decoration: none;
    color: #efefef;
}

a:hover {
    text-decoration: underline;
}

* {
    margin: 0;
    padding: 0;
}

wbr {
    display: inline-block;
}

html {
    background: url("blue/bg.png") #195287;
    color: #195287;
    font-size: 100%;
    height: 100%;
}

body {
    font-family: Helvetica, Verdana, Tahoma, Arial, 'Liberation Sans', FreeSans, sans-serif;
    font-size: .75em;
    position: relative;
    margin: 0 auto;
    min-height: 100%;
    padding: 20px 20px 0 20px;
    overflow: auto;
    width: 940px;
    _width: 945px; /* ie6 */
}

p {
    padding: 4px 10px 2px 10px;
    line-height: 1.6;
    text-align: left;
    vertical-align: top;
}

h1 {
    color: #fff;
    font-size: 170%;
    font-weight: bold;
    line-height: 1.5em;
    margin: 0 10px;
    _margin: 0 15px 0 10px; /* ie6 */
    padding: 10px 10px;
    text-align: center;
    text-shadow: #444 2px 2px 3px;
}

h2 {
    background-image: url('blue/title.png');
    background-color: #195287;
    color: #fff;
    font-size: 130%;
    font-weight: bold;
    line-height: 1.5em;
    padding: 3px 10px;
    /*text-transform: uppercase;*/
    text-shadow: #333 2px 2px 2px;
}

th, td, h3 {
    font-size: 100%;
    padding: 4px 10px 2px 10px;
    text-align: left;
    vertical-align: top;
}

table {
    width: 100%;
}

select {
    background-color: #fefefe;
}

#select {
    text-align: right;
    padding: 0 10px 0 10px;
    color: #fff
}

#select select {
    width: 100px;
}

.fullsize, .halfsize {
    background-color: #fff;
    float: left;
    margin: 10px 0 0 10px;
    _margin: 10px 5px 0 5px; /* ie6 */
    padding: 1px;
}

.fullsize {
    width: 918px;
}

.halfsize {
    width: 453px;
}

#filesystemTable thead tr th {
    cursor: pointer;
}

#footer {
    clear: both;
    color: #fff;
    margin: 12px;
    padding: 13px 25px;
    line-height: 18px;
    font-size: 80%;
    text-align: center;
}

.plugin {
    float: left;
    margin: 10px 0 0 10px;
    _margin: 10px 5px 0 5px; /* ie6 */
    padding: 1px;
    background-color: #f6f6f6;
}

.odd {
    background-color: #f6f6f6;
}

.even {
    background-color: #efefef;
}

.bar {
    background-image: url('blue/bar.png');
    background-color: #195287;
}

.barwarn {
    background-image: url('blue/barwarn.png');
    background-color: #871952;
}

.barrest {
    background-image: url('blue/barrest.png');
    background-color: #198785;
}

.right {
    padding-right: 20px;
    text-align: right;
}

.treeimg {
    display: table-cell;
    *float: left; /* ie6 ie7 */
    vertical-align: top;
}

.treespan {
    display: table-cell;
    *float: left; /* ie6 ie7 */
}

.treespanbold {
    font-weight: bold;
    display: table-cell;
    *float: left; /* ie6 ie7 */
}

.tree tr td {
    cursor: auto;
    height: 100%;
}

.treediv {
    display: table;
    height: 100%;
}

.tablemain {
    width: 100%;
    height: 100%;
}
